@@ -8,6 +8,11 @@ class PidFile(object):
         self.path = path
         self.pidfile = None
 
+    def close(self):
+        pidfile = self.pidfile
+        self.pidfile = None
+        pidfile.close()
+
     def __enter__(self):
         self.pidfile = open(self.path, "a+")
         fcntl.flock(self.pidfile.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
@@ -19,9 +24,16 @@ class PidFile(object):
         return self.pidfile
 
     def __exit__(self, exc_type=None, exc_value=None, exc_tb=None):
-        self.pidfile.close()
-        os.remove(self.path)
+        if self.pidfile:
+            self.pidfile.close()
+            os.remove(self.path)
 
+# compatibility later for swifclient < 2.7.0
+def smart_unicode(s, encoding='utf-8'):
+    if isinstance(s, unicode):
+        return s
+    else:
+        return unicode(s, encoding)
 
 #from django.utils
 def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
@@ -41,3 +53,4 @@ def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
         return s.decode('utf-8', errors).encode(encoding, errors)
     else:
         return s
+
